@@ -17,7 +17,7 @@ from . import common
 class Callback():
 
   @classmethod
-  def find_class(self, module_name: str) -> typing.Optional[typing.Any]:
+  def find_class(self, module_name: str, abort: bool = False) -> typing.Optional[typing.Any]:
     if common.VERBOSE:
       print("loading %s..." % module_name)
     try:
@@ -30,7 +30,10 @@ class Callback():
       return None
 
     except (ImportError, AttributeError):
-      print( f"Failed to load specific module: {sys.exc_info()[1]}")
+      if abort:
+        common.fatal(f"Failed to load specific module: {sys.exc_info()[1]}")
+      else:
+        print(f"Failed to load specific module: {sys.exc_info()[1]}")
       return None
 
   def call(self, name: str, *args: typing.Any, **kwargs: typing.Any) -> None:
